package com.zanxiang.game.back.serve.utils;

import com.github.sd4324530.jtuple.Tuple2;
import com.github.sd4324530.jtuple.Tuples;
import com.zanxiang.game.back.serve.pojo.entity.GameBackPolicy;
import org.apache.commons.lang3.RandomUtils;

import java.util.Arrays;
import java.util.Objects;
import java.util.function.Function;

public class BackPolicyUtil {
    private static final long[] RECHARGE_LEVEL = {12 * 100L, 30 * 100L, 50 * 100L, 98 * 100L, 198 * 100L, 328 * 100L, 649 * 100L, 998 * 100L, 2998 * 100L};

    static {
        Arrays.sort(RECHARGE_LEVEL);
    }

    /**
     * 订单回传
     *
     * @param gameBackPolicy   回传策略
     * @param rechargeMoney    充值金额
     * @param lastBackFunction 在指定最近订单数量内是否有订单回传过
     * @return <是否回传, 回传金额>
     */
    public static boolean backOrder(GameBackPolicy gameBackPolicy, long rechargeMoney, FirstOrder firstOrder, Function<Integer, Integer> lastBackFunction) {
        if (null == gameBackPolicy) {
            return true;
        }
        Integer firstType;
        String firstRate;
        Integer rechargeType;
        String rechargeRate;
        if (rechargeMoney <= gameBackPolicy.getFirstMinMoney()) {
            // 小于首充金额
            firstType = gameBackPolicy.getFirstMinMoneyType();
            firstRate = gameBackPolicy.getFirstMinMoneyRate();
        } else if (rechargeMoney >= gameBackPolicy.getFirstMaxMoney()) {
            firstType = gameBackPolicy.getFirstMaxMoneyType();
            firstRate = gameBackPolicy.getFirstMaxMoneyRate();
        } else {
            firstType = gameBackPolicy.getFirstOtherMoneyType();
            firstRate = gameBackPolicy.getFirstOtherMoneyRate();
        }
        if (rechargeMoney <= gameBackPolicy.getRechargeMinMoney()) {
            rechargeType = gameBackPolicy.getRechargeMinMoneyType();
            rechargeRate = gameBackPolicy.getRechargeMinMoneyRate();
        } else if (rechargeMoney >= gameBackPolicy.getRechargeMaxMoney()) {
            rechargeType = gameBackPolicy.getRechargeMaxMoneyType();
            rechargeRate = gameBackPolicy.getRechargeMaxMoneyRate();
        } else {
            rechargeType = gameBackPolicy.getRechargeOtherMoneyType();
            rechargeRate = gameBackPolicy.getRechargeOtherMoneyRate();
        }
        Integer backType;
        String backRate;
        if (firstType.equals(rechargeType) && firstRate.equalsIgnoreCase(rechargeRate)) {
            // 回传内容相同，不需要区分是不是首单
            backType = firstType;
            backRate = firstRate;
        } else {
            if (firstOrder.isFirstOrder()) {
                backType = firstType;
                backRate = firstRate;
            } else {
                backType = rechargeType;
                backRate = rechargeRate;
            }
        }
        if (Objects.equals(backType, GameBackPolicy.POLICY_TYPE_FIXED_RATE)) {
            // 固定比例回传
            String[] temp = backRate.split(":");
            int backCount = Integer.parseInt(temp[0]);
            int ignoreCount = Integer.parseInt(temp[1]);
            int isBackCount = lastBackFunction.apply(backCount + ignoreCount - 1);
            return isBackCount >= backCount;
        } else if (Objects.equals(backType, GameBackPolicy.POLICY_TYPE_RANDOM_RATE)) {
            // 随机概率回传
            return RandomUtils.nextInt(0, 100) <= Integer.parseInt(backRate);
        } else {
            throw new RuntimeException("回传策略[" + gameBackPolicy.getId() + "]配置错误，未知的回传类型：" + backType);
        }
    }

    private static Tuple2<Integer, Integer> approximate(int molecular, int denominator) {
        if (molecular % denominator == 0) {
            int temp = molecular / denominator;
            return Tuples.tuple(temp, temp);
        }
        int model = 2;
        do {
            if (molecular % model == 0 && denominator % model == 0) {
                molecular = molecular / model;
                denominator = denominator / model;
                continue;
            }
            model++;
        } while (model <= denominator / 2);
        return Tuples.tuple(molecular, denominator);
    }

    /**
     * 降档
     */
    public static long lowRechargeLevel(long rechargeMoney, Integer levelDown) {
        if (levelDown == null || levelDown == 0) {
            return rechargeMoney;
        }
        int level = RECHARGE_LEVEL.length - 1;
        for (int i = 0; i < RECHARGE_LEVEL.length; i++) {
            if (rechargeMoney <= RECHARGE_LEVEL[i]) {
                level = i;
                break;
            }
        }
        level = level - levelDown;
        level = level < 0 ? 0 : Math.min(level, RECHARGE_LEVEL.length - 1);
        return RECHARGE_LEVEL[level];
    }

    public interface FirstOrder {
        boolean isFirstOrder();
    }
}
